// *********************************************************************************
// ESSENTIAL INTERNAL ROUTINES
// *********************************************************************************

// =================================================================================
// 
// =================================================================================
function replaceHtml(el, html) {
	var oldEl = typeof el === "string" ? document.getElementById(el) : el;
	var newEl = oldEl.cloneNode(false);
	newEl.innerHTML = html;
	oldEl.parentNode.replaceChild(newEl, oldEl);
	return newEl;
};


function writeHTML(id, msg)
{
	replaceHtml(id, msg);
}
	

// =================================================================================
// Copy the contents of one HTML element into another
// The destination's previous contents are overwritten
// =================================================================================
function copyHTMLElement(source, destination)
{
	$(destination).html($(source).html());
}


// =================================================================================
// Move the contents of one HTML element into another
// The source's current contents are removed
// The destination's previous contents are overwritten
// =================================================================================
function moveHTMLElement(source, destination)
{
	$(destination).html($(source).detach().html());
	//$(source).appendTo(destination);
	/*var s = $(source);
	var h = s.html();
	s.empty();
	$(destination).html(h);*/
	//$(source).detach().html(destination);
	//$(destination).show()
}


// =================================================================================
// Append content of source div to content of destination div
// Since this uses jQuery, IDs need to use #
// The copied HTML content is wrapped in a div for convenience purposes
// =================================================================================
function appendHTMLElement(source, destination)
{
	$(destination).append("<div class=\"" + $(source).attr("id") + "\">" + $(source).html() + "</div>");
}


// *********************************************************************************
// GAME-SPECIFIC ROUTINES
// *********************************************************************************

var MAP_WIDTH = 64;
var MAP_HEIGHT = 75;
var TILE_SIZE = 4;

var currentMap = new Array(MAP_WIDTH * MAP_HEIGHT);
var currentFog = new Array(MAP_WIDTH * MAP_HEIGHT);
var currentPositionX = -1;
var currentPositionY = -1;

var mainDisplay;
var displayDrawContext;


var mapBuffer;
var mapBufferDrawContext;
var fogBuffer;
var fogBufferDrawContext;

var tilesetImage;

function init()
{
    tilesetImage = document.getElementById('tileset');
    
    mainDisplay = document.getElementById('mapDisplay');
    displayDrawContext = mapDisplay.getContext('2d');
    
    mapBuffer = document.createElement('canvas');
    mapBuffer.width = mainDisplay.width;
    mapBuffer.height = mainDisplay.height;
    mapBufferDrawContext = mapBuffer.getContext('2d');
    
    mapBufferDrawContext.fillStyle = 'black';
    mapBufferDrawContext.fillRect(0, 0, mapBuffer.width, mapBuffer.height);
    
    fogBuffer = document.createElement('canvas');
    fogBuffer.width = mainDisplay.width;
    fogBuffer.height = mainDisplay.height;
    fogBufferDrawContext = fogBuffer.getContext('2d');
    
    redrawDisplay();
}

function clearAllData()
{
	
}

function updateMapData(data)
{
    for (var idx = 0; idx < data.length; idx++)
    {
        if (idx >= currentMap.length)
        {
            break;
        }
        
        currentMap[idx] = data.charCodeAt(idx) % 16;
    }
    
    renderMap();
}

function updateFogData(data)
{
    for (var idx = 0; idx < data.length; idx++)
    {
        if (idx >= currentFog.length)
        {
            break;
        }
        
        currentFog[idx] = data.charCodeAt(idx) % 16;
    }
    
    renderAllFog();
}

function updatePosition(x, y)
{
    currentPositionX = x;
    currentPositionY = y;
    
    clearFogAt(x, y);
    
    redrawDisplay();
}

function setTile(x, y, value)
{
    var idx = y * MAP_WIDTH + x;
    
    if (idx >= 0 && idx < currentMap.length)
    {
        currentMap[idx] = value;
        
        _renderTileInternal(idx);
        redrawDisplay();
    }
}

function _renderTileInternal(idx)
{
    var x = idx % MAP_WIDTH;
    var y = Math.floor(idx / MAP_WIDTH);
    
    mapBufferDrawContext.drawImage(tilesetImage, currentMap[idx] * 16, 0, 16, 16, x * TILE_SIZE, y * TILE_SIZE, TILE_SIZE, TILE_SIZE);
}

function renderMap()
{
    for (var idx = 0; idx < currentMap.length; idx++)
    {
        _renderTileInternal(idx);
    }
    
    redrawDisplay();
}

function clearFogAt(x, y)
{
    var centerX = x * TILE_SIZE + TILE_SIZE / 2;
    var centerY = y * TILE_SIZE + TILE_SIZE / 2;
    var radius = TILE_SIZE * 5;
    
    var gradient = fogBufferDrawContext.createRadialGradient(centerX, centerY, 0, centerX, centerY, radius);
    gradient.addColorStop(0, 'rgba(0, 0, 0, .5)');
    gradient.addColorStop(1, 'rgba(0, 0, 0, 0)');
    
    fogBufferDrawContext.fillStyle = gradient;
    fogBufferDrawContext.fillRect(centerX - radius, centerY - radius, 2 * radius, 2 * radius);
    
    fogBufferDrawContext.fillStyle = 'black';
    fogBufferDrawContext.fillRect(x * TILE_SIZE, y * TILE_SIZE, TILE_SIZE, TILE_SIZE);
    
    redrawDisplay();
}

function renderAllFog()
{
    fogBufferDrawContext.clearRect(0, 0, fogBuffer.width, fogBuffer.height);
    
    for (var idx = 0; idx < currentMap.length; idx++)
    {
        var x = idx % MAP_WIDTH;
        var y = Math.floor(idx / MAP_WIDTH);
        
        if (currentFog[idx] < 8)
        {
            clearFogAt(x, y);
        }
    }
    
    redrawDisplay();
}

function redrawDisplay()
{
    displayDrawContext.globalCompositeOperation='source-over';
    displayDrawContext.drawImage(mapBuffer, 0, 0);
    
    displayDrawContext.globalCompositeOperation='destination-in';
    displayDrawContext.drawImage(fogBuffer, 0, 0);
    
    if (currentPositionX >= 0 && currentPositionY >= 0)
    {
        displayDrawContext.globalCompositeOperation='source-over';
        displayDrawContext.fillStyle = 'yellow';
        displayDrawContext.beginPath();
        displayDrawContext.arc(currentPositionX * TILE_SIZE + TILE_SIZE / 2, currentPositionY * TILE_SIZE + TILE_SIZE / 2, TILE_SIZE, 0, 2 * Math.PI, false);
        displayDrawContext.fill();
        displayDrawContext.strokeStyle = 'black';
        displayDrawContext.lineWidth = 2;
        displayDrawContext.stroke();
    }
}